
// const for progress bar
pub const TEMPLATE: &str = "[{prefix:>!20.green}] {bar:40.cyan/blue} {percent}% {binary_bytes_per_sec} [{elapsed_precise}]";
pub const PROGRESS_CHARS: &str = "#>-";
pub const FILE_READING: &str = "Reading file";
pub const TOKEN_GENERATING: &str = "Generating tokens";
pub const SPACE_GENERATING: &str = "Generating space";
